/***************************************************************************
 *   Copyright (C) 2004-2005 by Grzegorz Rusin                                  *
 *   pks@irc.pl                                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "header.h"

QCpuTemp::QCpuTemp(QWidget *parent, const char *name)
	: KSystemTray(parent, name)
{
	loadSettings();

	chart = new QChart(this, "cpu temperature", WDestructiveClose | WType_TopLevel | WStyle_Customize | WStyle_NoBorder);
	chart->resize(WIDTH, HEIGHT);
	chart->setFixedSize(WIDTH, HEIGHT);
	if(showGraph) chart->show();

	temp = new cArray<int>(PROBES);

	temp->push(readTemp(THRM_ZONE));
	temp->push(readTemp(THRM_ZONE));

	chart->update(temp);

	startTimer(UPDATE_INTERVAL);

	QTimer *iconTimer = new QTimer(this);
	iconTimer->startTimer(5000);
	connect(iconTimer, SIGNAL(timeout()), this, SLOT(drawIcon()));

	QToolTip::add(this, "CPU thermal zone monitor");

	/*
	binPath.sprintf("/proc/%d/cmdline", getpid());
	QFile cmdline(binPath);
	cmdline.open(IO_ReadOnly);
	cmdline.readLine(binPath, 4096);
	*/
}

void QCpuTemp::setBinPath(QString string)
{
	binPath = string;
}

void QCpuTemp::timerEvent(QTimerEvent *)
{
	temp->push(readTemp(THRM_ZONE));
	chart->update(temp);
}

void QCpuTemp::mousePressEvent(QMouseEvent *e)
{
	if(e->button() == Qt::LeftButton)
	{
		if(chart->isHidden()) chart->show();
		else if(chart->isMinimized()) chart->showNormal();
		else if(chart->isShown()) chart->hide();
	}
	else
	{

		KIconLoader icon;

		KPopupMenu *menu = new KPopupMenu(this);
		menu->insertTitle(mainIcon, "cpu temp");

		menu->insertItem(icon.loadIconSet("text", KIcon::Small), "Set &tray font",  this, SLOT(setTrayFont()), CTRL+Key_T);

		if(QFile(QString("%1/.kde/Autostart/cputemp").arg(getenv("HOME"))).exists())
			menu->insertItem(icon.loadIconSet("remove", KIcon::Small), "Remove from autostart", this, SLOT(removeFromAutostart()));
		else
			menu->insertItem(icon.loadIconSet("ok", KIcon::Small), "Add to autostart", this, SLOT(addToAutostart()));

		menu->insertItem(icon.loadIconSet("info", KIcon::Small), "&About",  about, SLOT(show()), CTRL+Key_A);
		menu->insertSeparator();
		menu->insertItem(icon.loadIconSet("exit", KIcon::Small), "&Quit", this, SLOT(closeApp()));
		menu->exec(QCursor::pos());
		delete menu;
	}
}

void QCpuTemp::drawIcon()
{
	QPixmap icon;
	QBitmap mask;
	QPainter paint;
	QColor color;
	QPen pen;

	icon.resize(size());
	mask.resize(size());

	mask.fill(Qt::color0);

	int t = readTemp(THRM_ZONE);

	paint.begin(&mask);
	paint.setPen(Qt::color1);
	paint.setFont(trayFont);
	paint.drawText(0, 0, width()-1, height()-1, AlignCenter, QString("%1").arg(t));
	paint.end();

	paint.begin(&icon);
	paint.setFont(trayFont);

	if(t <= 30) color.setHsv(120, 255, 255);
	else if(t >= 70) color.setHsv(0, 255, 255);
	else color.setHsv(120-(t-30)*((255-120)/40),255, 255);

	paint.setPen(color);
	paint.drawText(0, 0, width()-1, height()-1, AlignCenter, QString("%1").arg(t));
	paint.end();

	icon.setMask(mask);
	setPixmap(icon);


	mainIcon.resize(16, 16);
	mainIcon.fill(chart->backgroundColor);
	paint.begin(&mainIcon);

	QPointArray p;
	p.resize(6);
	p.setPoint(0, -1, 11);
	p.setPoint(1,  3,  7);
	p.setPoint(2,  7, 11);
	p.setPoint(3, 20, -1);
	p.setPoint(4, 16, 16);
	p.setPoint(5, -1, 16);

	QBrush brush(chart->graphColor, Qt::Dense4Pattern);
	paint.setBrush(brush);
	pen.setWidth(0);
	pen.setColor(chart->backgroundColor);
	paint.setPen(pen);
	paint.drawPolygon(p, TRUE);
}

void QCpuTemp::saveSettings()
{
	QSettings settings;

	settings.insertSearchPath(QSettings::Windows, WINDOWS_REGISTRY);
	settings.writeEntry(APP_KEY + "trayFont", trayFont.toString());
	settings.writeEntry(APP_KEY + "showGraph", chart->isShown());
}

void QCpuTemp::loadSettings()
{
	QSettings settings;

	settings.insertSearchPath(QSettings::Windows, WINDOWS_REGISTRY);
	trayFont.fromString(settings.readEntry(APP_KEY + "trayFont", "Helvetica,14,-1,5,50,0,0,0,0,0"));
	showGraph = settings.readBoolEntry(APP_KEY + "showGraph", TRUE);
}


QCpuTemp::~QCpuTemp()
{
	delete temp;
	saveSettings();
}

void QCpuTemp::contextMenuAboutToShow(KPopupMenu *menu)
{
	if(menu->count() == 3)
	{
		menu->insertItem("Set &tray font",  this, SLOT(setTrayFont()), CTRL+Key_T, 1, 1);
		menu->insertItem("&About",  about, SLOT(show()), CTRL+Key_A, 2, 2);
	}
}

void QCpuTemp::setTrayFont()
{
	bool ok;
	QFont tmp = trayFont;

	tmp = QFontDialog::getFont(&ok, tmp, this );

	if(ok)
	{
		trayFont = tmp;
		drawIcon();
	}
}

void QCpuTemp::resizeEvent(QResizeEvent *)
{
	drawIcon();
}

int QCpuTemp::readTemp(const char *from)
{
	char buf[32];
	FILE *f = fopen(from, "r");

	if(!f) return -1;

	int temp;
	fscanf(f, "%32s %d", buf, &temp);
	fclose(f);
	return temp;
}

void QCpuTemp::closeApp()
{
	exit(0);
}

void QCpuTemp::removeFromAutostart()
{
	unlink(QString("%1/.kde/Autostart/cputemp").arg(getenv("HOME")));
}

void QCpuTemp::addToAutostart()
{
	QString lnk;
	lnk.sprintf("%s/.kde/Autostart/cputemp", getenv("HOME"));
	unlink(lnk);
	symlink(binPath, lnk);
}
