/***************************************************************************
 *   Copyright (C) 2004-2005 by Grzegorz Rusin                                  *
 *   pks@irc.pl                                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef HEADER_H
#define HEADER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "cArray.h"
#include "about.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

#include <qobject.h>
#include <qwidget.h>
#include <qnamespace.h>
#include <qapplication.h>
#include <qpixmap.h>
#include <qpointarray.h>
#include <qtimer.h>
#include <qpainter.h>
#include <qcolor.h>
#include <qcolordialog.h>
#include <qcursor.h>
#include <qpopupmenu.h>
#include <qpen.h>
#include <qfontdialog.h>
#include <qfont.h>
#include <qframe.h>
#include <qsettings.h>
#include <qbitmap.h>
#include <qlcdnumber.h>
#include <qtooltip.h>
#include <qfile.h>
#include <qiconset.h>

#include <kapplication.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <ksystemtray.h>
#include <kpopupmenu.h>
#include <kiconloader.h>
#include <kicontheme.h>

#define PX_PER_DEG		5
#define PX_PER_PROBE 	3
#define PROBES 			180
#define RANGE			70
#define UPDATE_INTERVAL	60000
#define THRM_ZONE		"/proc/acpi/thermal_zone/THRM/temperature"

#define WIDTH			PROBES*PX_PER_PROBE
#define HEIGHT			RANGE*PX_PER_DEG

const QString WINDOWS_REGISTRY = "/pks/cputemp";
const QString APP_KEY = "/cputemp/";

class QSimpleButton : public QWidget
{
	Q_OBJECT
	QPixmap *_pixmap;

	public:
	QSimpleButton(QWidget *parent = 0, const char *name = 0, WFlags f = 0)
			: QWidget(parent, name, f) { _pixmap = NULL; };
	~QSimpleButton() { if(_pixmap) delete _pixmap; };
	void mousePressEvent(QMouseEvent *e) { if(e->button() == Qt::LeftButton) emit clicked(); };
	void setPixmap(QPixmap &px) { _pixmap = &px; repaint(FALSE); };
	void paintEvent(QPaintEvent *) { bitBlt(this, 0, 0, _pixmap, 0, 0, width(), height(), Qt::CopyROP); };
	QPixmap *pixmap() { return _pixmap; };

	signals:
	void clicked();
};

class QChart : public QWidget
{
	Q_OBJECT
	QPixmap bb;

	QFont textFont;
	QPointArray *points;
	QPoint lastMousePos;
	QSimpleButton *hideButton, *minimizeButton;
	bool drag;

	private:
	void redrawHideButton();
	void redrawMinimizeButton();

	public:
	QColor backgroundColor, gridColor, graphColor, textColor;

	QChart(QWidget *parent = 0, const char *name = 0, WFlags f = 0);
	~QChart();
	void paintEvent(QPaintEvent *e);
	void resizeEvent(QResizeEvent *e);
	void drawGrid(QPainter *p=NULL);
	void update(cArray<int> *array=NULL);
	void mousePressEvent(QMouseEvent *e);
	void mouseMoveEvent(QMouseEvent *e);
	void mouseReleaseEvent(QMouseEvent *e);
	int lastProbe();

	public slots:
	void setGridColor();
	void setBackgroundColor();
	void setGraphColor();
	void setTextColor();
	void setTextFont();
	void saveSettings();
	void loadSettings();

	signals:
	void newData();
};

class QCpuTemp : public KSystemTray
{
	Q_OBJECT
	QPoint lastChartPos;
	QChart *chart;
	QFont trayFont;
	QLCDNumber *lcd;
	cArray<int> *temp;
	int i;
	bool showGraph;
	QString binPath;
	QPixmap mainIcon;

	public:
	QAboutDialog *about;
	QCpuTemp(QWidget *parent = 0, const char *name = 0);
	~QCpuTemp();

	void timerEvent(QTimerEvent *e);
	void mousePressEvent(QMouseEvent *e);
	void contextMenuAboutToShow(KPopupMenu *menu);
	void resizeEvent(QResizeEvent *e);
	void setBinPath(QString path);


	public slots:
	void drawIcon();
	void loadSettings();
	void saveSettings();
	void setTrayFont();
	static int readTemp(const char *from);
	void closeApp();
	void addToAutostart();
	void removeFromAutostart();
};

#endif // HEADER_H
